import platform

from DrissionPage import ChromiumPage, ChromiumOptions
from loguru import logger
from scan.common import ProjectInfo


def get_free_ip():
    url = "https://www.zdaye.com/free/?ip=&adr=&checktime=&sleep=&cunhuo=&dengji=1&nadr=&https=&yys=&post=&px="
    browser.get(url, retry=3, interval=1, timeout=15)
    ip_ports = []
    for tr in browser.eles('x://table[@id="ipc"]//tr')[1:]:
        tds = [td.text for td in tr.eles("x://td")]
        ip_ports.append((f"{tds[0]}:{tds[1]}", tds[3]))
    print(len(ip_ports), ip_ports)
    return ip_ports


def switch_ip(ip_port=None):
    global set_proxy
    if ip_port:
        # 设置proxy

        ip, port = ip_port.split(":")
        print("设置代理", ip, port)
        tab = browser.new_tab()
        tab.get("chrome-extension://padekgcemlokbadohgkifijomclgjgif/options.html#!/profile/proxy")
        tab.ele('x://input[@ng-model="proxyEditors[scheme].host"]').input(ip, clear=True)
        tab.ele('x://input[@ng-model="proxyEditors[scheme].port"]').input(port, clear=True)
        tab.ele('x://a[@ng-click="applyOptions()"]').click()
        tab.close()

        tab = browser.new_tab()
        tab.get("chrome-extension://padekgcemlokbadohgkifijomclgjgif/popup/index.html#")
        tab.ele('x://span[text()="proxy"]').click()
        tab.close()
        # 提示框
        # txt = tab.handle_alert()
        # print("提示框", txt)
        # tab.handle_alert(accept=False)
        # if not omega_proxy:
        #     # 切换proxy
        #     tab.get("chrome-extension://padekgcemlokbadohgkifijomclgjgif/popup/index.html#")
        #     # tab.wait(1)
        #     tab.ele('x://span[text()="proxy"]').click()
        #     set_proxy = True
        #     tab.close()
    else:
        print("直接链接")
        tab = browser.new_tab()
        tab.get("chrome-extension://padekgcemlokbadohgkifijomclgjgif/popup/index.html#")
        tab.ele('x://span[text()="[直接连接]"]').click()
        tab.close()
    # if len(browser.tab_ids) > 1:
    #     print("当前tab个数", len(browser.tab_ids))
    #     tab.close()
    # tab.close()


if platform.system().lower() == 'windows':  # 这里判断是不是windows,具体地址自己改
    co = ChromiumOptions()  # .set_paths(browser_path=r"C:\Program Files (x86)\Microsoft\Edge\Application\msedge.exe")
else:
    co = ChromiumOptions().set_paths(browser_path=r"/opt/google/chrome/google-chrome")
    co.headless(True)  # 设置无头加载  无头模式是一种在浏览器没有界面的情况下运行的模式，它可以提高浏览器的性能和加载速
    # co.incognito(True)  # 无痕隐身模式打开的话，不会记住你的网站账号密码的
    co.set_argument(
        '--no-sandbox')  # 禁用沙箱 禁用沙箱可以避免浏览器在加载页面时进行安全检查,从而提高加载速度 默认情况下，所有Chrome 用户都启用了隐私沙盒选项  https://zhuanlan.zhihu.com/p/475639754
    co.set_argument("--disable-gpu")  # 禁用GPU加速可以避免浏览器在加载页面时使用过多的计算资源，从而提高加载速度
    co.set_user_agent(
        user_agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36')  # 设置ua

# co.set_timeouts(6, 6, 6)
# co.set_local_port(9211)
# 1、设置switchyOmega插件
# co.add_extension(r'C:\Users\Ad\Desktop\proxy_switchyomega-2.5.20')
proxy_switch_extension_path = ProjectInfo.DOCS_DIR_PATH / "proxy_switchyomega-2.5.20"
co.add_extension(proxy_switch_extension_path)
browser = ChromiumPage(co)

# 2、重置switchyOmega插件
# omega_proxy = False
# switch_ip()

# 3、随机切换代理ip
# ip_all = get_free_ip()
# ip_all = [{"ip": "10.1.3.56", "port": 7890, "expire_time": "2024-04-27 22:24:00"}]
ip_all = [{"ip": "127.0.0.1", "port": 7897, "expire_time": "2024-04-27 22:24:00"}]
for ips in ip_all:
    logger.info(f"~~~切换ip，now {ips['ip']}")
    # 重置switchyOmega插件
    switch_ip(f"{ips['ip']}:{ips['port']}")
    # switch_ip()
    try:
        browser.get("https://www.ipaddress.my/?lang=zh_CN", retry=0)
        logger.success(f">>>>>>>>切换代理成功 {browser.html}")
    except Exception as err:
        logger.error(f"----------切换代理失败 dp {err}")

# browser.quit()




import json
from hashlib import md5

from scan import crawl, logger


def product(url):

    proxy = "http://127.0.0.1:7890"
    resp = crawl.open(
        "https://www.crocs.com/",
        # "https://rucaptcha.com/42",
        # "https://www.perplexity.ai/",
        # "https://milled.com/pricing",
        # "https://nopecha.com/demo/cloudflare",
        timeout=10,
        is_cloudflare=True,
        load_mode="none"
    )
    print("111", resp.response.cookies)
    # resp2 = crawl.open(
    #     "https://www.perplexity.ai",
    #     is_cloudflare=True
    # )
    # print("222", resp2.response.cookies)
    # resp3 = crawl.open(
    #     "https://nopecha.com/demo/cloudflare",
    #     is_cloudflare=True
    # )
    # print("333", resp3.response.cookies)
    # print("333 text", resp3.text())
    # print(resp.text())
    # print(resp2.response.cookies)
    # print(resp3.response.cookies)
    # crawl.close_dp()
    # print("22222",  resp.text())
    # resp = await crawl.fetch(url)
    crawl.close_dp()


if __name__ == "__main__":
    product("https://www.perplexity.ai/")




from scan import logger, crawl
import time
import random
import asyncio
from scan.common import Settings

Settings.dp_request_wait_times = 5
Settings.dp_max_tabs_count = 3


def dp_crawl():
    while True:
        try:
            # crawl.create_dp(
            #     # headless=True,
            #     # local_port=9222
            # )
            crawl.open("https://github.com", load_mode="none")
            # crawl.clear_dp_cache(cache=True, cookies=True)
            resp = crawl.open(
                "https://rucaptcha.com/42",
                # proxies="127.0.0.1:7897",
                is_cloudflare=True,
                load_mode="none",
            )
            print(resp.status_code, resp.response.url)
            # resp = crawl.open(url=url, is_cloudflare=True, load_mode="none")
            _code = resp.status_code
            real_url = resp.response.url
            text = resp.text()
            history = {'code': _code, 'url': real_url}
            print(_code, real_url, text[:20], history)

            logger.debug(f"resp cookie: {resp.response.cookies}")
            logger.debug(f"resp text: {resp.text()[:1000]}")
            resp2 = crawl.open(
                "https://www.perplexity.ai", is_cloudflare=True, load_mode="none"
            )
            logger.debug(f"resp2 cookie: {resp2.response.cookies}")
            resp3 = crawl.open(
                url="https://nopecha.com/demo/cloudflare", is_cloudflare=True, load_mode="none"
            )
            logger.debug(f"resp3 cookie: {resp3.response.cookies}")
            logger.debug(f"resp3 text: {resp3.text()[:1000]}")
        except Exception as e:
            logger.debug(f"error: {str(e)}")
            # raise
        time.sleep(3)

    # crawl.close_dp()


async def tdp(url):
    crawl.create_dp()
    crawl.open(url, load_mode="none", is_cloudflare=True)
    # crawl.close_dp()


async def task():
    logger.info("开始执行任务...")
    task_list = []
    urls = [
        "https://rucaptcha.com/42",
        "https://github.com",
        "https://www.baidu.com",
        "https://www.google.com",
        "https://www.apple.com",
    ]
    for _ in urls:
        print("___", _)
        t = asyncio.create_task(tdp(_))
        task_list.append(t)
    await asyncio.gather(*task_list)
    logger.info("结束执行任务...")


if __name__ == "__main__":
    # asyncio.run(consume_image_tasks())
    asyncio.run(task())




from threading import Lock
from typing import Literal, NamedTuple, Optional, Union

from DrissionPage import ChromiumOptions, ChromiumPage
from tenacity import retry, retry_if_result, stop_after_attempt, wait_random

from scan.common import ProjectInfo, Settings, logger
from scan.response import Response


def create_retry_decorator():
    """用于解决 retry 无法动态设置重试参数的问题"""
    max_attempts = int(Settings.dp_request_wait_times)

    return retry(
        stop=stop_after_attempt(max_attempts),
        wait=wait_random(
            min=Settings.dp_wait_min_seconds, max=Settings.dp_wait_max_seconds
        ),
        retry=retry_if_result(lambda x: any([x.response.err_msg == "too many tabs"])),
    )


class DrissionpageResponse(NamedTuple):
    content: Optional[bytes] = None
    status_code: Optional[int] = None
    text: Optional[str] = None
    cookies: Optional[dict] = None
    err_msg: Optional[str] = None
    url: Optional[str] = None


class DrissionpageSingletonMeta(type):
    _instance = None
    _lock = Lock()

    def __call__(cls, *args, **kwargs):
        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    cls._instance = super().__call__(*args, **kwargs)
        return cls._instance


class DrissionpageDownloader(metaclass=DrissionpageSingletonMeta):
    def __init__(self):
        self.page = None
        self.driver = None

    def create_dp(
        self,
        headless: bool = True,
        user_agent: str = ProjectInfo.default_user_agent,
        no_imgs: bool = False,
        proxy: Optional[str] = None,
        browser_path: Optional[str] = None,
        local_port: int = 9222,
    ):
        if browser_path is None:
            co = ChromiumOptions()
        else:
            co = ChromiumOptions(browser_path)
        co.set_timeouts(base=Settings.co_set_timeouts)
        if local_port:
            co.set_local_port(local_port)
        co.set_argument("--no-sandbox")
        if no_imgs:
            co.no_imgs(True)
        if headless:
            co.headless(True)
        if user_agent:
            co.set_user_agent(user_agent)

        turnstile_extension_path = ProjectInfo.DOCS_DIR_PATH / "turnstilePatch"
        proxy_switch_extension_path = (
            ProjectInfo.DOCS_DIR_PATH / "proxy_switchyomega-2.5.20"
        )
        co.add_extension(turnstile_extension_path)
        co.add_extension(proxy_switch_extension_path)
        if proxy:
            co.set_proxy(proxy)
        self.driver = ChromiumPage(addr_or_opts=co)

    def connect_dp(self, local_port: int = 9222):
        self.driver = ChromiumPage(addr_or_opts=local_port)

    def switch_ip(self, ip_port=None):
        if ip_port:
            # 设置代理
            ip, port = ip_port.split(":")
            tab = self.driver.new_tab()
            tab.get(
                "chrome-extension://padekgcemlokbadohgkifijomclgjgif/options.html#!/profile/proxy"
            )
            tab.ele('x://input[@ng-model="proxyEditors[scheme].host"]').input(
                ip, clear=True
            )
            tab.ele('x://input[@ng-model="proxyEditors[scheme].port"]').input(
                port, clear=True
            )
            tab.ele('x://a[@ng-click="applyOptions()"]').click()
            tab.close()

            # 开启代理
            tab = self.driver.new_tab()
            tab.get(
                "chrome-extension://padekgcemlokbadohgkifijomclgjgif/popup/index.html#"
            )
            tab.ele('x://span[text()="proxy"]').click()
            tab.close()
        else:
            # 直接连接
            tab = self.driver.new_tab()
            tab.get(
                "chrome-extension://padekgcemlokbadohgkifijomclgjgif/popup/index.html#"
            )
            tab.ele('x://span[text()="[直接连接]" or text()="[Direct]"]').click()
            tab.close()

    def turnstile(self, url, headers=None, is_cloudflare=False):
        if headers:
            self.page.set.headers(headers=headers)
        self.page.get(url=url)
        if is_cloudflare:
            for i in range(25):
                try:
                    curr_title = self.page.title
                    if all(
                        ["just a moment" not in curr_title.lower(), "请稍候" not in curr_title, "لحظات" not in curr_title]
                    ):
                        break

                    challengeSolution = self.page.ele("@name=cf-turnstile-response")
                    challengeWrapper = challengeSolution.parent()
                    challengeIframe = challengeWrapper.shadow_root.ele("tag:iframe")
                    challengeIframeBody = challengeIframe.ele("tag:body").shadow_root
                    challengeButton = challengeIframeBody.ele("tag:input")
                    challengeButton.click()
                except Exception as e:
                    # 这里会因为没有找到元素而报错，不用管，元素可能还没有渲染出来，遍历查询即可
                    pass

        content = self.page.html
        cookie = self.get_cookie()
        self.page.close()
        return content, cookie

    def get_cookie(self) -> dict:
        return self.page.cookies

    def request(
        self,
        url,
        headers=None,
        proxies: Optional[Union[str, Literal[False]]] = None,
        timeout=30,
        retry=1,
        show_errmsg=False,
        interval=2,
        session=True,
        load_mode="normal",
        clear_cache=False,
        is_cloudflare=False,
        connect_dp=True,
    ):
        @create_retry_decorator()
        def inner_request():
            if connect_dp:
                if ProjectInfo.is_windows:
                    self.create_dp()
                else:
                    try:
                        self.connect_dp()
                    except Exception as e:
                        self.create_dp()

            response = Response()
            response.request_url = url
            response.response = DrissionpageResponse(
                err_msg="too many tabs",
                url=url,
            )
            tabs_count = self.driver.tabs_count
            if tabs_count >= Settings.dp_max_tabs_count:
                logger.info(f"标签页超过 {Settings.dp_max_tabs_count} 个，重试等待中。")
                return response

            if proxies:
                self.switch_ip(proxies)
            # 当关闭代理时配置直接连接
            if proxies is False:
                self.switch_ip()

            self.page = self.driver.new_tab()
            if load_mode == "none":
                self.page.set.load_mode.none()
            elif load_mode == "eager":
                self.page.set.load_mode.eager()
            if clear_cache:
                self.clear_cache(cache=True, cookies=False)
            content, cookies = self.turnstile(
                url=url, headers=headers, is_cloudflare=is_cloudflare
            )
            response.response = DrissionpageResponse(
                content=bytes(content, encoding="utf-8"),
                status_code=200,
                text=content,
                cookies=cookies,
                url=url,
            )
            return response

        return inner_request()

    def close_dp(self):
        if self.driver:
            self.driver.quit(force=False)
            logger.info("DrissionPage closed")

    def clear_cache(self, cache=True, cookies=True):
        if self.driver:
            self.driver.clear_cache(cache, cookies)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hscan",
    version="4.1.1",
    author="jyanghe",
    author_email="jyanghe1023@gmail.com",
    description="A python framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jyangHe/hscan",
    packages=setuptools.find_packages(),
    include_package_data=True,
    package_data={
        'scan': ['**/*'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8.0",
    install_requires=[
        "httpx[http2]>=0.27.2",
        "requests",
        "aiofiles==0.7.0",
        "aio-pika==6.8.0",
        "beautifulsoup4==4.9.3",
        "aioredis==2.0.0",
        "motor==2.5.1",
        "Brotli==1.0.9",
        "pymongo==3.12.0",
        "chardet==4.0.0",
        "asyncpg==0.26.0",
        "boto3==1.33.13",
        "aiobotocore==2.9.0",
        "aiomysql==0.1.1",
        "curl-cffi==0.5.9",
        "aiohttp==3.8.6",
        "loguru",
        "oss2==2.15.0",
        "aiokafka==0.8.0",
        "DrissionPage==4.0.2",
        "tenacity==9.0.0",
    ],
    setup_requires=["wheel"],
)



from typing import (TYPE_CHECKING, Any, AsyncIterable, Iterable, Literal,
                    Optional, Union)

from scan.common import ProjectInfo, logger
from scan.downloade.aiohttp_downloader import Downloader as AioHttpDownloader
from scan.downloade.downloader import Downloader
from scan.downloade.dp_downloader import DrissionpageDownloader

if TYPE_CHECKING:
    from http.cookiejar import CookieJar

    RequestContent = Union[str, bytes, Iterable[bytes], AsyncIterable[bytes]]


class Crawl:
    def __init__(self):
        self.downloader = Downloader()
        self.aiohttp_downloader = AioHttpDownloader()
        self.dp_downloader = None

    async def fetch(self,  url, params=None, data=None, files=None, json=None, content=None, headers=None, cookies=None,
                    verify=True, http2=False, auth=None, proxies=None, allow_redirects=True, stream=False,
                    session=False, timeout=30, cycle=1, tls=False, use_aiohttp=False):
        if use_aiohttp:
            res = await self.aiohttp_downloader.request(
                url, params=params, data=data, files=files, json=json, content=content, headers=headers,
                proxies=proxies,
                verify=verify, http2=http2, cookies=cookies, auth=auth, allow_redirects=allow_redirects,
                timeout=timeout,
                cycle=cycle, stream=stream, tls=tls, session=session
            )
        else:
            res = await self.downloader.request(
                url, params=params, data=data, files=files, json=json, content=content, headers=headers,
                proxies=proxies, verify=verify, http2=http2, cookies=cookies, auth=auth,
                allow_redirects=allow_redirects, timeout=timeout, cycle=cycle, stream=stream, tls=tls, session=session
            )
        return res

    def create_dp(
        self,
        headless: bool = True,
        user_agent: str = ProjectInfo.default_user_agent,
        no_imgs: bool = False,
        browser_path: Optional[str] = None,
        proxy: Optional[str] = None,
        local_port: int = 9222,
    ):
        self.dp_downloader = DrissionpageDownloader()
        self.dp_downloader.create_dp(
            headless=headless,
            user_agent=user_agent,
            no_imgs=no_imgs,
            proxy=proxy,
            browser_path=browser_path,
            local_port=local_port,
        )
        logger.info("DrissionPage created")

    def open(
        self,
        url: str,
        params: Optional[dict] = None,
        data: Union[dict, str] = None,
        files: Any = None,
        json: Union[dict, str] = None,
        content: "RequestContent" = None,
        headers: Optional[dict] = None,
        cookies: Union[dict, "CookieJar"] = None,
        verify: bool = True,
        http2: bool = False,
        auth: Any = None,
        proxies: Optional[Union[str, Literal[False]]] = None,
        allow_redirects: bool = True,
        stream: bool = False,
        session=False,
        timeout: Optional[Union[int, float]] = None,
        cycle: int = 1,
        tls: bool = False,
        use_aiohttp: bool = False,
        show_errmsg: bool = False,
        interval: Union[int, float] = 2,
        hooks: Any = None,
        load_mode: Literal["normal", "eager", "none"] = "normal",
        clear_cache: bool = False,
        is_cloudflare: bool = False,
        connect_dp: bool = True,
    ):
        if not self.dp_downloader:
            logger.info("未先使用 crawl.creat_dp 创建 DrissionPage，将使用默认设置！")
            self.dp_downloader = DrissionpageDownloader()
            self.dp_downloader.create_dp()

        res = self.dp_downloader.request(
            url=url,
            headers=headers,
            proxies=proxies,
            timeout=timeout,
            retry=cycle,
            show_errmsg=show_errmsg,
            interval=interval,
            load_mode=load_mode,
            clear_cache=clear_cache,
            is_cloudflare=is_cloudflare,
            connect_dp=connect_dp,
        )
        return res

    def close_dp(self):
        self.dp_downloader.close_dp()

    def clear_dp_cache(self, cache=True, cookies=True):
        self.dp_downloader.clear_cache(cache, cookies)

    async def close(self):
        await self.downloader.close_all()


crawl: Crawl = Crawl()
__all__ = crawl



import os
import platform
import sys
from pathlib import Path

from scan.log import get_logger


class ProjectInfo:
    ROOT_DIR_PATH = Path(__file__).parent
    DOCS_DIR_PATH = ROOT_DIR_PATH / "docs"

    is_linux = platform.system().lower() == "linux"
    is_macos = platform.system().lower() == "darwin"
    is_windows = platform.system().lower() == "windows"

    if is_linux:
        _ua_platform_id = "X11; Linux x86_64"
    elif is_macos:
        _ua_platform_id = "Macintosh; Intel Mac OS X 10_15_7"
    elif is_windows:
        _ua_platform_id = "Windows NT 10.0; Win64; x64"
    else:
        _ua_platform_id = "X11; Linux x86_64"
    default_user_agent = f"Mozilla/5.0 ({_ua_platform_id}) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/130.0.0.0 Safari/537.36"


class Settings:
    # dp 排队请求重试次数
    dp_request_wait_times = 100
    # dp 排队请求重试时间设置
    dp_wait_min_seconds = 1
    dp_wait_max_seconds = 2
    # dp 最大标签页
    dp_max_tabs_count = 4
    co_set_timeouts = 1


if ProjectInfo.is_windows:
    sn = "hscan"
else:
    try:
        file_path = os.path.abspath(sys.argv[0])
        sn = file_path.split("/")[-1].replace(".py", "")
    except:
        sn = "hscan"

logger = get_logger(sn)



from DrissionPage import ChromiumPage, ChromiumOptions
import time


def turnstile_challenge(page):
    for i in range(6):
        try:
            curr_title = page.title
            if all(
                ["just a moment" not in curr_title.lower(), "请稍候" not in curr_title]
            ):
                break

            print(f"第 {i + 1} 次尝试")
            if i == 0:
                time.sleep(2)
            else:
                time.sleep(0.5)

            challengeSolution = page.ele(
                "@name=cf-turnstile-response", timeout=1
            )
            challengeWrapper = challengeSolution.parent()
            challengeIframe = challengeWrapper.shadow_root.ele(
                "tag:iframe", timeout=1
            )
            challengeIframeBody = challengeIframe.ele(
                "tag:body", timeout=1
            ).shadow_root
            challengeButton = challengeIframeBody.ele("tag:input", timeout=1)
            challengeButton.focus()
            challengeButton.click()
        except Exception:
            pass


while True:
    try:
        co = ChromiumOptions()
        co.set_local_port(9223)
        co.headless(True)
        co.set_user_agent("Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/130.0.0.0 Safari/537.36")
        co.set_argument('--no-sandbox')
        co.set_argument("--disable-gpu")
        driver = ChromiumPage(co)
        driver.clear_cache(cache=True, cookies=True)

        page = driver.new_tab()
        page.get("https://www.dogalecza.com.tr/selfish-drop-cilt-bakimi-2913?ttclid=E_C_P_CscBCl2ZJ5wkz8R7QIa1piD55ZM8JYXCDoOa7wRFOHpqjQs3txTqaEkaQxXL5uYUNmmeRvb58jEdsKvmu0v23IVK1CF84csU2VTLa23pObxOzm5kzUkH7oW7nZzSBl9u8aTQgRN5xd427Mg0XDVsPxj5If5Tw2utodBUcY_nVohK-73lsz9wzgwKblGPOqZizZjapSASoPy0XDtfyYFJF4t2luQy6q0FVp16UPSXbmkk-XZOKdTCDz9e1yiZU3jP5bEdtJ7DxX26NhIEdjIuMBog-8OcfL6JTr7MfEutOZBFg-_iBYQ8jx5c26VUrSHYyaM")
        turnstile_challenge(page)
        print(f"page 的标题为：{page.title}")
        page.close()
    except Exception as e:
        if "无法操作当前存储数据" in str(e):
            pass
        else:
            print(str(e))
            raise
